module Quandl
  module Utility
    class Configuration
      class << self
        def define_attributes(*attribute_names)
          attribute_names.each do |key|
            define_attribute(key)
          end
        end

        def attributes
          @attributes ||= []
        end

        protected

        def define_attribute(key)
          key = key.to_s
          attributes << key unless attributes.include?(key)
          define_method(key) { read_attribute(key) }
          define_method("#{key}=") { |value| write_attribute(key, value) }
        end
      end

      def initialize(object = {})
        self.attributes = object
        after_initialize
      end

      def after_initialize
      end

      def to_h
        attributes
      end

      def attributes=(attrs)
        attrs.each do |key, value|
          send("#{key}=", value) if self.respond_to?("#{key}=")
        end
      end

      def attributes
        @attributes ||= {}
      end

      protected

      def read_attribute(key)
        attributes[key.to_s]
      end

      def write_attribute(key, value)
        attributes[key.to_s] = value
      end
    end
  end
end
