require 'rubocop'
require 'thor'
require 'thor/foodcritic'

module Quandl
  module Utility
    module ThorTasks
      # Linting Tasks
      class Lint < Thor
        namespace 'qutil:lint'
        include Thor::Actions

        def self.source_root
          File.expand_path('../../../../../support/templates', __FILE__)
        end

        desc 'cookbook', 'stuff'
        def cookbook
          invoke 'foodcritic:lint'
        end

        method_option :paths,
          type: :array,
          default: %w( . ),
          required: true,
          aliases: '-p'
        desc 'ruby', 'Lint project for Ruby style with Rubocop'
        def ruby
          result = RuboCop::CLI.new.run(options[:paths])
          if result == 0
            say('No Rubocop warnings', :green)
          else
            exit(result)
          end
        end

        method_option :default_directory,
          type: :string,
          default: '.quandl.rubocop',
          aliases: '-d',
          required: true,
          banner: 'DIRECTORY'
        desc 'init_ruby', 'Start Rubocop linting for project'
        def init_ruby
          return unless yes?('Do you want to setup Rubocop linting?', :yellow)
          # Get directory and file for base rubocop
          dir = options[:default_directory]
          file_path = File.join(dir, '.rubocop.yml')
          # Create empty directory if it does not exist
          empty_directory(dir)
          # Create base rubocop with common rules. Let Thor handle conflicts.
          template('quandl.rubocop.yml', file_path)
          # Create project template. Let Thor handle conflicts
          @base_directory = dir
          template('project.rubocop.yml.erb', '.rubocop.yml')
        end
      end
    end
  end
end
